package video;

import futils.Futil;
import gui.In;
import gui.dialogs.ProgressDialog;
import gui.run.RunJob;
import ip.color.Octree;
import ip.gif.gifAnimation.Gif89Encoder;
import ip.gif.neuquantAnimation.AnimatedGifEncoder;
import j2d.ImageUtils;
import j2d.ShortImageBean;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;


/**
 * DocJava, Inc. http://www.docjava.com Programmer: dlyon Date: Nov 23,
 * 2004 Time: 1:12:41 PM
 */
public class ScreenCapture {
    Vector v = new Vector();
    RunJob rj = null;
    Octree ot = null;

    public static void grabAndSaveAnImage() {
        try {

            grabAndSaveAnImage(In.getRectangle("select a capture area"),
                    Futil.getWriteFile("select a PNG output file"));
            In.message("saved");
        } catch (AWTException e) {
            e.printStackTrace();

        } catch (IOException e) {
            e.printStackTrace();

        }
    }

    public static void grabAndSaveGifAnimation() {
        ScreenCapture p = new ScreenCapture();
        System.out.println("Recording");
        int k = In.getInt("enter the number of frames to capture");
        p.record(k);    // number of seconds per frame
        System.out.println("images saved");
    }

    public static void grabAndSaveAnImage(Rectangle rect,
                                          File outputFile)
            throws AWTException, IOException {
        // create screen shot
        Robot robot = new Robot();
        BufferedImage image = robot.createScreenCapture(rect);
        // save captured image to PNG file
        ImageIO.write(image, getImageFormat(), outputFile);

    }

    public static String getImageFormat() {
        return (String) In.multiPrompt(ImageIO.getWriterFormatNames(),
                "select an output format", "format selector dialog");
    }

    private void record(int k) {
        for (int i = 0; i < k; i++)
            grabScreen();
        saveNeuquantImages();
    }

    public void snapAndSaveOneFrame() {
        grabScreen();
        saveScreen();
    }

    private void saveScreen() {
        Futil.setSwing(false);

        File f = Futil.getWriteFile("select and output file (gif)");
        ImageUtils.saveAsGif(makeGray(ImageUtils.getImage(
                (BufferedImage) v.elementAt(0))),
                f);

    }

    /**
     * @param img
     * @return a gray image
     */
    private Image makeGray(Image img) {
        ShortImageBean sib = new ShortImageBean(img);
        sib.gray();
        return sib.getImage();
    }

    /**
     * reduce the number of colors in an image sequence.
     *
     * @param NumberOfColors
     * @return an array of color reduced images.
     */
    private Image[] getColorReducedImages(int NumberOfColors) {
        ot = new Octree();
        final int n = v.size();
        Image imgs[] = new Image[n];
        System.out.println("begin color reduction process");
        ProgressDialog pd = new ProgressDialog();
        int incr = 200 / n;
        pd.setSize(100, 100);
        pd.show();
        for (int i1 = 0; i1 < n; i1++) {
            pd.incrementValue(incr);
            ShortImageBean sib1 = new ShortImageBean(
                    ImageUtils.getImage((BufferedImage) v.elementAt(i1)));
            if (i1 == 0) {
                ot.octreeQuantization(sib1.getR(),
                        sib1.getB(),
                        sib1.getG(), NumberOfColors);
                continue;
            }
            ot.addImagesSeen(sib1.getR(), sib1.getB(), sib1.getG());

        }
        System.out.println("done with lut");
        for (int i = 0; i < n; i++) {
            ShortImageBean sib = new ShortImageBean(
                    ImageUtils.getImage((BufferedImage) v.elementAt(i)));
            ot.reMap(sib.getR(), sib.getG(), sib.getB());
            sib.swapGreenAndBlue();
            imgs[i] = sib.getImage();
            pd.incrementValue(incr);
        }
        System.out.println("colors reduced in image sequence");
        return imgs;

    }


    private void saveNeuquantImages()  {
        File f = Futil.getWriteFile("select gif output file");
        AnimatedGifEncoder age = new AnimatedGifEncoder();
        System.out.println("v.size="+v.size());
        age.setDelay(1000);
        In.message("writing worked=" + age.start(f.toString()));
        final int n = v.size();
        for (int i=0; i < n; i++) {
           boolean b= age.addFrame((BufferedImage)v.elementAt(i));
            if (b==false) In.message("problem adding image");
        }

        age.finish();
    }
    private void saveImages() throws IOException {
        File f = Futil.getWriteFile("select gif output file");
        int speed = 2; //fps
        //Image img [] = getImages();
        Image img[] = getColorReducedImages(240);
        int n = img.length;
        if (n == 0) return;
        FileOutputStream fos =
                Futil.getFileOutputStream(f);
        Gif89Encoder ge = new Gif89Encoder();
        for (int i = 1; i < n; i++) {
            ge.addFrame(img[i]);
            ge.setUniformDelay(Math.round(100 / speed));
            System.out.println("processed frame:" + i);
        }
        System.out.println("done!");
        ge.setLoopCount(true ?
                0 :
                1);
        ge.encode(fos);
        fos.close();
    }

    public void grabScreen() {
        v.addElement(getImage());
    }

    private static BufferedImage getImage() {
        try {
            return ImageUtils.captureScreen();
        } catch (AWTException e) {
            e.printStackTrace();
        }
        return null;
    }

}